function [A,S,mu_a,mu_s, varexp,cost]=ConvSC(X,noc,lags,lambda1,sparseFreq,opts)
%
% Algorithm for Convolutive Sparse Coding described in
% Kristoffer Hougaard Madsen, Lars Kai Hansen and Morten Mrup "Combining Time and Frequency Domain
% Optimization: Shifts, Convolution and Smoothness in Factor Analysis Type
% Decompositions"
%
% Please refer to this article when publishing results obtained by the
% algortihm.
%
% Usage:
%   [A,S,mu_a,mu_s,varexp,cost]=ConvSC(X,noc,lags,lambda1,sparseFreq,opts)
%
% Input:
% X                 I x J matrix
% noc               number of components
% lags              number of time lags in A
% lambda1           L1-regularization strength on S
% sparseFreq        0: Impose sparsity in time domain (default)
%                   1: impose sparsity in frequency domain.
% opts. 
%    A              Initial A
%    S              Initial S
%    maxiter        maximum number of iteration (default: 1000)
%    conv_crit      Convergence Criterion (default relative change of costfunction of 1e-6)
%    mu_a           Initial step size A update
%    mu_s           Initial step size S update
%
% Output:
% A             I x noc x lags matrix
% S             noc x J matrix of sources
% mu_a          Final A step size
% mu_s          Final S step size
% varexp        Variation explained by the model
% cost          Final objective function value
%
% Copyright (C) Morten Mrup, Kristoffer Hougaard Madsen and Technical University of Denmark, 
% February 2008

if nargin<6
    opts=struct;
end
if nargin<5
    sparseFreq=0;
end

% Initialize variables

[M,N]=size(X);
if isfield(opts,'S')
    S=opts.S;
else
    S=randn(noc,N);
end
if isfield(opts,'A')
    A=opts.A;
else
    A=randn(M,noc,lags);
end

for k=1:noc
    A(:,k,:)=A(:,k,:)/sqrt(sum(sum(A(:,k,:).^2,1),3));
end
mod2=~mod(N,2);
Sf=fft(S,[],2);
N2=floor(N/2+1);
Sf=Sf(:,1:N2);
Xf=fft(X,[],2);
Xf=Xf(:,1:N2);
mu_s=mgetopt(opts,'mu_s',1e-2);
mu_a=mgetopt(opts,'mu_a',1e-4);

% Initialize alg parameters
Af=zeros(M,noc,N);
Af(:,:,1:size(A,3))=A;
Af=fft(Af,[],3);        
Af=Af(:,:,1:N2);
Recf=squeeze(sum(conj(Af).*permute(repmat(Sf,[1,1,M]),[3 1 2]),2));
if sparseFreq
    regcost=lambda1*sum(abs(Sf(:)))/N;
else
    regcost=lambda1*sum(abs(S(:)));
end
cost=0.5/N*CostLS(Xf,Recf,mod2)+regcost;
conv_crit=mgetopt(opts,'conv_crit',1e-6);
maxiter=mgetopt(opts,'maxiter',500);
dcost=inf;
iter=0;
SST=sum(X(:).^2);
dheader = sprintf('%12s | %12s | %12s | %12s | %12s | %12s | %12s','Iteration','Expl. var.','Cost func.','Delta costf.',' Time(s)   ',' A-stepsize',' S-stepsize');
dline = sprintf('-------------+--------------+--------------+--------------+--------------+--------------+--------------+');
  

while dcost>conv_crit & iter<maxiter
    tic;
    if mod(iter,10)==0
        disp(dline);
        disp(dheader);
        disp(dline);
    end
    costold2=cost;
    costold=cost;
    iter=iter+1;      
    
    % update S    
    gradSf=squeeze(sum(Af.*permute(repmat(Recf-Xf,[1,1,noc]),[1 3 2]),1));
    stop=0;
    if sparseFreq
        if mod2
            gradSf(:,[2:end-1])=2*gradSf(:,[2:end-1]);    
        else
            gradSf(:,2:end)=2*gradSf(:,2:end);    
        end
        Sf_old=Sf;
        while ~stop % Perform linesearch
            Sf=Sf_old-mu_s*gradSf;
            Sf(abs(Sf)<mu_s*lambda1)=0;
            Sf=Sf-lambda1*mu_s*Sf./(abs(Sf)+eps);
            Recf=squeeze(sum(conj(Af).*permute(repmat(Sf,[1,1,M]),[3 1 2]),2));
            regcost=lambda1*sum(abs(Sf(:)))/N;
            cost=0.5/N*CostLS(Xf,Recf,mod2);
            if cost+regcost<=costold
                stop=1;
                mu_s=mu_s*1.2;
            else
                mu_s=mu_s/2;
            end
        end               
    else % sparse in time domain
        gradS=ifft_nyquist(gradSf,mod2);  
        S_old=S;
        while ~stop % Perform linesearch
            S=S_old-mu_s*gradS;
            S(abs(S)<mu_s*lambda1)=0;
            S=S-lambda1*mu_s*sign(S);
            Sf=fft(S,[],2);
            Sf=Sf(:,1:N2);
            Recf=squeeze(sum(conj(Af).*permute(repmat(Sf,[1,1,M]),[3 1 2]),2));
            regcost=lambda1*sum(abs(S(:)));
            cost=0.5/N*CostLS(Xf,Recf,mod2);
            if cost+regcost<=costold
                stop=1;
                mu_s=mu_s*1.2;
            else
                mu_s=mu_s/2;
            end
        end
    end
    costold=cost;
    
    % Update A
    gradAf=permute(repmat(Sf,[1,1,M]),[3 1 2]).*permute(repmat(conj(Recf-Xf),[1,1,noc]),[1 3 2]);    
    Aold=A;
    stop=0;
    gradA=ifft_nyquist(gradAf,mod2);
    gradA=gradA(:,:,1:lags);
    gradA=gradA-repmat(sum(sum(gradA.*A,1),3),[M,1,lags]).*A;
    while ~stop % Perform linesearch
        A=Aold-mu_a*gradA;
        for k=1:noc
            A(:,k,:)=A(:,k,:)/sqrt(sum(sum(A(:,k,:).^2,1),3));
        end
        Af=zeros(M,noc,N);
        Af(:,:,1:size(A,3))=A;
        Af=fft(Af,[],3);
        Af=Af(:,:,1:N2);
        Recf=squeeze(sum(conj(Af).*permute(repmat(Sf,[1,1,M]),[3 1 2]),2));
        cost=0.5/N*CostLS(Xf,Recf,mod2);
        if cost<=costold
            stop=1;
            mu_a=mu_a*1.2;
        else
            mu_a=mu_a/2;
        end             
    end
    varexp=1-2*cost/SST;
    cost=cost+regcost;    
    dcost=(costold2-cost)/cost;
    disp(sprintf('%12.0f | %12.4f | %12.4f | %12.4f | %12.4f | %2.4e | %2.4e ',iter,varexp,cost,dcost ,toc,mu_a,mu_s));
end
if sparseFreq
    S=ifft_nyquist(Sf,mod2);
end

    
%--------------------------------------------------------------------------
function cost=CostLS(Xf,Recf,mod2)
% Calculate LS error
if mod2
   cost=norm(Xf(:,[1 end])-Recf(:,[1 end]),'fro')^2+2*norm(Xf(:,[2:end-1])-Recf(:,[2:end-1]),'fro')^2;
else
   cost=norm(Xf(:,1)-Recf(:,1),'fro')^2+2*norm(Xf(:,[2:end])-Recf(:,[2:end]),'fro')^2;
end

%--------------------------------------------------------------------------
function H=ifft_nyquist(Hf,mod2)
% The inverse fourier transform of a real signal given by frequencies up to
% Nyquist
% Input
%   Hf      noc x F matrix of component frequencies up to the Nyquist
%           criterion
%   mod2    1: even number of samples, 0: odd number of samples
%
% Output
%   H       noc x J matrix in time domain

if ndims(Hf)==2
    if mod2
        Hf=[Hf conj(Hf(:,end-1:-1:2))];
        H=ifft(Hf,[],2,'symmetric');
    else
        Hf=[Hf conj(Hf(:,end:-1:2))];
        H=ifft(Hf,[],2,'symmetric');
    end
elseif ndims(Hf)==3
    if mod2
        Hf=cat(3,Hf,conj(Hf(:,:,end-1:-1:2)));
        H=ifft(Hf,[],3,'symmetric');
    else
        Hf=cat(3,Hf,conj(Hf(:,:,end:-1:2)));
        H=ifft(Hf,[],3,'symmetric');
    end
end



% -------------------------------------------------------------------------
% Parser for optional arguments
function var = mgetopt(opts, varname, default, varargin)
if isfield(opts, varname)
    var = getfield(opts, varname); 
else
    var = default;
end
for narg = 1:2:nargin-4
    cmd = varargin{narg};
    arg = varargin{narg+1};
    switch cmd
        case 'instrset',
            if ~any(strcmp(arg, var))
                fprintf(['Wrong argument %s = ''%s'' - ', ...
                    'Using default : %s = ''%s''\n'], ...
                    varname, var, varname, default);
                var = default;
            end
        otherwise,
            error('Wrong option: %s.', cmd);
    end
end

