function [W, H, T, varexpl, cost]=ShiftNMF(X,noc,lambda1,lambda2,lambda3,opts)
%
% algorithm for Shifted Non-negative Matrix factorization described in
% Morten Mrup, Kristoffer Hougaard Madsen and Lars Kai Hansen " Shifted
% Non-negative Matrix Factorization", submitted to MLSP2007.
% Please refer to this article when publishing results obtained by the
% algortihm.
%
% Usage:
% [W, H, T, varexpl]=shiftNMF(X,noc,lambda1,lambda2,lambda3,opts)
%
% Input:
% X                 I x J matrix
% noc               number of components
% lambda1           L1-regularization strength on H (default 0)
% lambda2           L2-regularization strength on H (default 0)
% lambda3           smoothness strengh on H         (default 0, can be unstable if lambda3>0)
% opts. 
%    W              Initial W
%    H              Initial H
%    T              Initial T
%    freqweight     Weighting of the smoothness over frequencies of H 
%                   (default ones(noc,size(floor(X/2)+1,2)));
%    maxiter        maximum number of iteration (default: 1000)
%    conv_crit      Convergence Criterion (default relative change of costfunction of 1e-6)
%    ConstW         1: Constant W 
%    ConstH         1: Constant H
%    ConstT         1: Constant T
%    alpha          initial stepsize of H update
%    dispiter       1: Display result for iteration  
%   
%
% Output:
% W             I x noc mixing matrix 
% H             noc x J matrix of sources
% T             I x noc matrix of time delays
%     
% varexpl   vector of Variation explained for ShiftNMF solutions through
%           the iterations
% cost      vector of cost function value through the iterations

%
% Copyright (C) Morten Mrup, Kristoffer Hougaard Madsen and Technical University of Denmark, 
% April 2007

if nargin<6
    opts=struct;
end
if nargin<5
    lambda3=0;
end
if nargin<4
    lambda2=0;
end
if nargin<3
    lambda1=0;
end
  
[I J]=size(X);
mod2=~mod(J,2);
costold=inf;
runit=mgetopt(opts,'runit',0);
if ~isfield(opts,'H') & runit~=1 
    disp(['Finding the best out of ' num2str(5) ' initial solution'])
    for k=1:5
        if k==1
            disp(['Now Estimating ' num2str(k) 'st solution ']);
        elseif k==2
            disp(['Now Estimating ' num2str(k) 'nd solution ']);
        elseif k==3
            disp(['Now Estimating ' num2str(k) 'rd solution ']);
        else
            disp(['Now Estimating ' num2str(k) 'th solution ']);
        end
        optsn=opts;
        optsn.dispiter=0;
        optsn.runit=1;    
        optsn.maxiter=25;
        [Wq, Hq, Tq, varexpl,cost]=ShiftNMF(X,noc,lambda1,lambda2,lambda3,optsn);
        if cost(end)<costold
            costold=cost(end);
            disp(['Lowest cost-function ' num2str(costold) ]);
            W=Wq;
            H=Hq;
            T=Tq;
        end
    end
else
    mx=max(abs(X(:)));
    W=mgetopt(opts,'W',mx*rand(I,noc));
    H=mgetopt(opts,'H',mx*rand(noc,J));
    T=mgetopt(opts,'T',zeros(size(W)));
end
W=W./repmat(sqrt(sum(W.^2,1)),[I,1]);
 
nyT=mgetopt(opts,'nyT',1);
maxiter=mgetopt(opts,'maxiter',1000);
conv_crit=mgetopt(opts,'convcrit',1e-6);
SST=norm(X,'fro')^2;
constH=mgetopt(opts,'constH',0);
constW=mgetopt(opts,'constW',0);
constT=mgetopt(opts,'constT',0);
freqweight=mgetopt(opts,'freqweight',ones(noc,floor(J/2)+1));
estWholeSamples=mgetopt(opts,'estWholeSamples',1);

if size(freqweight,1)==1
    freqweight=repmat(freqweight,noc,1);      
end
auto_corr=mgetopt(opts,'auto_corr',1);
smoothtype=mgetopt(opts,'smoothtype','none');
smoothnorm=mgetopt(opts,'smoothnorm','L2');
alpha=mgetopt(opts,'alpha',1);
dispiter=mgetopt(opts,'dispiter',1);

f=j*2*pi*[0:J-1]/J;
f_full=f;
Xf=fft(X,[],2);
Xf=Xf(:,1:floor(J/2)+1);
Hf=fft(H,[],2);
Hf=Hf(:,1:floor(J/2)+1);
f=f(1:size(Xf,2));

% Initial Cost
regcost=lambda3*smoothCost(Hf,freqweight,mod2)/J+lambda1*sum(H(:))+lambda2/2*sum(H(:).^2);
costold=LScostf(Xf,W,Hf,T,f,mod2)/J+regcost;
varexpl=(SST-2*(costold-regcost))/SST;
cost=costold;

dcost=inf;
told=cputime;
iter=0;

% Display algorithm progress
if dispiter
    disp([' '])
    disp(['Shifted Non-negative matrix factorization'])
    disp(['A ' num2str(noc) ' component model will be fitted']);
    disp(['To stop algorithm press control C'])
    dheader = sprintf('%12s | %12s | %12s | %12s | %12s | %12s | %12s','Iteration','Expl. var.','Cost func.','Delta costf.',' Time(s)   ',' H-stepsize');
    dline = sprintf('-------------+--------------+--------------+--------------+--------------+--------------+');
    disp(dline);
    disp(dheader);
    disp(dline);
end


while iter<maxiter & (dcost>=cost*conv_crit | mod(iter,20)==0 ) 

    iter=iter+1;        
    if mod(iter,10)==0 & dispiter
        disp(dline);
        disp(dheader);
        disp(dline);
    end
        
    % Update H
    if ~constH
        gradnH=zeros(size(Hf));
        gradpH=zeros(size(Hf));
        for i=1:size(Hf,2)
            Wf=(W.*exp(T*f(i)));
            gradnH(:,i)=Wf'*Xf(:,i);
            gradpH(:,i)=Wf'*Wf*Hf(:,i);
        end
        gradnH=ifft_nyquist(gradnH,mod2);
        gradpH=ifft_nyquist(gradpH,mod2);
        
        if lambda3>0
            if mod2
                Hft=[Hf conj(Hf(:,end-1:-1:2))];
                freqweight_t=[freqweight conj(freqweight(:,end-1:-1:2))];
            else
                freqweight_t=[freqweight conj(freqweight(:,end-1:-1:2))];
                Hft=[Hf conj(Hf(:,end:-1:2))];
            end
            sH=ifft(Hft.*freqweight_t,[],2,'symmetric');
            sHp=zeros(size(sH));
            sHn=sHp;
            ind=(sH>0);
            sHp(ind)=sH(ind);
            ind=(sH<0);
            sHn(ind)=-sH(ind);   
        else
            sHn=0;
            sHp=0;
        end
        gradnH(gradnH<0)=0;
        gradpH(gradpH<0)=0;
        grad=(gradnH+eps+lambda3*sHn)./(gradpH+lambda1+lambda2*H+lambda3*sHp+eps);
        Hold=H;
        Recf=zeros(size(Xf));
        keepgoing=1;
        while keepgoing
             H=Hold.*(grad.^alpha);
             Hf=fft(H,[],2);
             Hf=Hf(:,1:floor(size(Hf,2)/2)+1);
             smcost=smoothCost(Hf,freqweight,mod2)/J;
             regcost=lambda1*sum(H(:))+lambda2/2*sum(H(:).^2)+lambda3*smcost;
             cost=LScostf(Xf,W,Hf,T,f,mod2)/J+regcost;
             if cost>costold
                 alpha=alpha/2;
                 keepgoing=1;
             else
                 alpha=alpha*1.2;
                 keepgoing=0;
             end
         end
    end
    % Update W and calculate cost function
     if ~constW 
        for i=1:size(W,1)
           Hft=Hf.*exp(T(i,:)'*f); 
           Ht=ifft_nyquist(Hft,mod2);
           Ht(Ht<0)=0;
           gradnW(i,:)=X(i,:)*Ht';
           gradpW(i,:)=W(i,:)*(Ht*Ht');
        end
        tx = sum(gradpW.*W,1);
        ty = sum(gradnW.*W,1);
        gradnW = gradnW + repmat(tx,[size(W,1),1]).*W;
        gradpW = gradpW + repmat(ty,[size(W,1),1]).*W;                                                
        W=W.*(gradnW)./(gradpW+eps);
        W=W./repmat(sqrt(sum(W.^2,1)),[size(W,1),1]);
    end

    % Update T and calculate cost    
    if ~constT
        if estWholeSamples
            T=estTimeAutCor(Xf,W,Hf,T,f,J,mod2);
            cost=LScostf(Xf,W,Hf,T,f,mod2)/J+regcost;        
        else
            P.W=W;
            P.Xf=Xf;
            P.Hf=Hf;
            P.J=size(X,2);
            P.w=ones(1,length(f));
            if mod(size(X,2),2)==0
                P.w(2:end-1)=2;
            else
                P.w(2:end)=2;
            end
            P.f=f;
            P.nyT=nyT;
            [T,nyT,cost]=update_TmH(T,P);
            cost=cost/J+regcost; % Use Parseval identity for cost function
        end
    else
        cost=LScostf(Xf,W,Hf,T,f,mod2)/J+regcost;        
    end
    sse=2*(cost-regcost);
    
    % Display output of iteration
    dcost=costold-cost;
    costold=cost;
    costiter(iter)=cost;
    varexpl(iter)=(SST-sse)/SST;
    if rem(iter,1)==0 & dispiter
        t=cputime;
        tim=t-told;
        told=t;
        disp(sprintf('%12.0f | %12.4f | %12.4f | %4.5e | %12.4f | %12.4f ',iter, varexpl(iter),cost,dcost/cost,tim,alpha));
    end

end
cost=costiter;

% Align H and T
if runit==0
    tmean=mean(T);
    T=T-repmat(tmean,[size(T,1),1]);
    Hf=Hf.*exp(tmean'*f);
    H=ifft_nyquist(Hf,mod2);
    H(H<0)=0;
end
if estWholeSamples && runit==0
    opts.estWholeSamples=0;
    opts.T=T;
    opts.W=W;
    opts.H=H;
    [W, H, T, varexpl, cost]=ShiftNMF(X,noc,lambda1,lambda2,lambda3,opts);
end



% -------------------------------------------------------------------------
% Parser for optional arguments
function var = mgetopt(opts, varname, default, varargin)
if isfield(opts, varname)
    var = getfield(opts, varname); 
else
    var = default;
end
for narg = 1:2:nargin-4
    cmd = varargin{narg};
    arg = varargin{narg+1};
    switch cmd
        case 'instrset',
            if ~any(strcmp(arg, var))
                fprintf(['Wrong argument %s = ''%s'' - ', ...
                    'Using default : %s = ''%s''\n'], ...
                    varname, var, varname, default);
                var = default;
            end
        otherwise,
            error('Wrong option: %s.', cmd);
    end
end

%--------------------------------------------------------------------------
function H=ifft_nyquist(Hf,mod2)
% The inverse fourier transform of a real signal given by frequencies up to
% Nyquist
% Input
%   Hf      noc x F matrix of component frequencies up to the Nyquist
%           criterion
%   mod2    1: even number of samples, 0: odd number of samples
%
% Output
%   H       noc x J matrix in time domain

if mod2
    Hf=[Hf conj(Hf(:,end-1:-1:2))];
    H=ifft(Hf,[],2,'symmetric');
else
    Hf=[Hf conj(Hf(:,end:-1:2))];
    H=ifft(Hf,[],2,'symmetric');
end

%--------------------------------------------------------------------------
function cost=LScostf(Xf,W,Hf,T,f,mod2)
    Recf=zeros(size(Xf));
    for i=1:size(W,1)
        Hft=Hf.*exp(T(i,:)'*f); 
        Recf(i,:)=W(i,:)*Hft;
    end    
    if mod2
        cost=(0.5*norm(Xf(:,[1 end])-Recf(:,[1 end]),'fro')^2+norm(Xf(:,[2:end-1])-Recf(:,[2:end-1]),'fro')^2);
    else
        cost=(0.5*norm(Xf(:,1)-Recf(:,1),'fro')^2+norm(Xf(:,[2:end])-Recf(:,[2:end]),'fro')^2);
    end

%--------------------------------------------------------------------------   
function smcost=smoothCost(Hf,freqweight,mod2)
    Hf=(Hf.*freqweight).*conj(Hf);
    if mod2
        smcost=0.5*sum(sum(Hf(:,[1 end])))+sum(sum(Hf(:,[2:end-1])));
    else
        smcost=0.5*sum(sum(Hf(:,1)))+sum(sum(Hf(:,[2:end])));
    end


% -------------------------------------------------------------------------
function [T,W]=estTimeAutCor(Xf,W,Hf,T,f,J,mod2)
noc=size(W,2);
Xft=Xf;
t1=randperm(size(Xf,1));
t2=randperm(noc);
for k=t1
    for d=t2
        nocm=setdiff(1:noc,d);
        Xft(k,:)=Xf(k,:)-W(k,nocm)*(Hf(nocm,:).*exp(T(k,nocm)'*f(1,:)));
        C=(conj(Xft(k,:)).*Hf(d,:));
        C=ifft_nyquist(C,mod2);
        [y,ind]=max(C);
        T(k,d)=(ind-J)-1;
        W(k,d)=C(ind)/((sum(2*(Hf(d,:).*conj(Hf(d,:))))-sum(Hf(d,[1 end]).*conj(Hf(d,[1 end]))))/J);
        if abs(T(k,d))>(J/2)
            if T(k,d)>0
                T(k,d)=T(k,d)-J;
            else 
                 T(k,d)=T(k,d)+J;
            end
        end                  
    end
end


% -------------------------------------------------------------------------
function [T,nyT,cost]=update_TmH(T,P)
    nyT=P.nyT;
    Hf=P.Hf;
    W=P.W;
    J=P.J;
    Xf=P.Xf;
    f=P.f;
    w=P.w;
    Recfd=zeros(size(W,1),size(Hf,2),size(W,2));
    for d=1:size(W,2)
        Recfd(:,:,d)=(repmat(W(:,d),[1 length(f)]).*exp(T(:,d)*f)).*repmat(Hf(d,:),[size(W,1),1]);
    end
    Recf=sum(Recfd,3);        
    Q=Recfd.*repmat(conj(Xf-Recf),[1,1,size(Recfd,3)]);
    Hdiag=2*squeeze(sum(repmat((w.*(f.^2)),[size(Q,1),1,size(Q,3)]).*real(Q),2));    
    Hall=zeros(size(W,1),size(W,2),size(W,2));
    for d=1:size(W,2)
        Hall(:,:,d)=2*squeeze(sum(repmat((w.*(f.^2)),[size(Q,1),1,size(Q,3)]).*real(Recfd.*repmat(conj(Recfd(:,:,d)),[1,1,size(W,2)])),2));
    end    
    grad=squeeze(sum(repmat((w.*f),[size(Q,1),1,size(Q,3)]).*(conj(Q)-Q),2)); 
    for i=1:size(grad,1)
        ind=find(abs(grad(i,:))>1e-6);
        grad(i,ind)=grad(i,ind)/(-diag(Hdiag(i,ind))-squeeze(Hall(i,ind,ind)));
    end

    ind1=find(w==2); % Areas used twice
    ind2=find(w==1); % Areas used once
    cost_old=norm(Xf(:,ind1)-Recf(:,ind1),'fro')^2;
    cost_old=cost_old+0.5*norm(Xf(:,ind2)-Recf(:,ind2),'fro')^2;
    keepgoing=1;
    Told=T;
    while keepgoing
        T=Told-nyT*grad;
        for d=1:size(W,2)
            Recfd(:,:,d)=(repmat(W(:,d),[1 length(f)]).*exp(T(:,d)*f)).*repmat(Hf(d,:),[size(W,1),1]);
        end
        Recf=sum(Recfd,3);   
        cost=norm(Xf(:,ind1)-Recf(:,ind1),'fro')^2;
        cost=cost+0.5*norm(Xf(:,ind2)-Recf(:,ind2),'fro')^2;
        if cost<=cost_old
            keepgoing=0;
            nyT=nyT*1.2;
        else
            keepgoing=1;
            nyT=nyT/2;
        end
    end
    T=mod(T,J);
    ind=find(T>floor(J/2));
    T(ind)=T(ind)-J;
    
    
