function [A,S,mu_a,mu_s,varexp,cost]=SmoothSC(X,noc,lambda1,lambda2,opts)

%
% Algorithm for Smooth Sparse Matrix Factorization / Smooth Sparse Coding,
% The algorithm is described in
% Kristoffer Hougaard Madsen, Lars Kai Hansen and Morten Mrup "Combining Time and Frequency Domain
% Optimization: Shifts, Convolution and Smoothness in Factor Analysis Type
% Decompositions"
%
% Please refer to this article when publishing results obtained by the
% algortihm.
%
% Usage:
%   [A,S,mu_a,mu_s,,varexp,cost]=SmoothSC(X,noc,lambda1,lambda2)
%
% Input:
% X                 I x J matrix
% noc               number of components
% lambda1           L1-regularization strength on S
% lambda2           noc x floor(J/2)+1 matrix of weight in frequency domain
%
% opts. 
%    A              Initial A
%    S              Initial S
%    maxiter        maximum number of iteration (default: 1000)
%    conv_crit      Convergence Criterion (default relative change of costfunction of 1e-6)
%    mu_a           Initial step size A update
%    mu_s           Initial step size S update
%
% Output:
% A             I x noc x lags matrix
% S             noc x J matrix of sources
% mu_a          Final A step size
% mu_s          Final S step size
% varexp        Variation explained by the model
% cost          Final objective function value
%
% Copyright (C) Morten Mrup, Kristoffer Hougaard Madsen and Technical University of Denmark, 
% February 2008

if nargin<5
    opts=struct;
end

% Initialize variables
[M,N]=size(X);
S=mgetopt(opts,'S',randn(noc,N));
A=mgetopt(opts,'A',randn(M,noc));
for k=1:noc
    A(:,k)=A(:,k)/sqrt(sum(A(:,k).^2,1));
end
mod2=~mod(N,2);
Sf=fft(S,[],2);
N2=floor(N/2+1);
if nargin<4
    lambda2=zeros(noc,N2);
end
lambda2w=2*lambda2;
if mod2
    lambda2w(:,[1 end])=lambda2(:,[1 end]);
else
    lambda2w(:,1)=lambda2(:,1);
end
Sf=Sf(:,1:N2);
Xf=fft(X,[],2);
Xf=Xf(:,1:N2);
mu_s=mgetopt(opts,'mu_s',1e-2);
mu_a=mgetopt(opts,'mu_a',1e-4);
conv_crit=mgetopt(opts,'conv_crit',1e-6);
maxiter=mgetopt(opts,'maxiter',500);

% Initialize alg parameters
SST=sum(X(:).^2);
AtA=A'*A;
cost=SST/2+sum(sum((AtA).*(S*S')))/2-sum(sum(X.*(A*S)))+lambda1*sum(abs(S(:)))+0.5*sum(sum(lambda2w.*(Sf.*conj(Sf))))/N;
dcost=inf;
iter=0;
dheader = sprintf('%12s | %12s | %12s | %12s | %12s | %12s | %12s','Iteration','Expl. var.','Cost func.','Delta costf.',' Time(s)   ',' A-stepsize',' S-stepsize');
dline = sprintf('-------------+--------------+--------------+--------------+--------------+--------------+--------------+');
  

while dcost>conv_crit & iter<maxiter
    tic;
    if mod(iter,10)==0
        disp(dline);
        disp(dheader);
        disp(dline);
    end
    costold2=cost;
    costold=cost;
    iter=iter+1;      
    
    % update S  
    AtX=A'*X;
    gradS=AtA*S-AtX+ifft_nyquist(lambda2.*Sf,mod2);
    S_old=S;
    stop=0;
    while ~stop % Perform linesearch
        S=S_old-mu_s*gradS;
        S(abs(S)<mu_s*lambda1)=0;
        S=S-lambda1*mu_s*sign(S);
        Sf=fft(S,[],2);
        Sf=Sf(:,1:N2);
        SSt=S*S';
        regcost=lambda1*sum(abs(S(:)))+0.5*sum(sum(lambda2w.*(Sf.*conj(Sf))))/N;
        cost=SST/2+sum(sum(AtA.*SSt))/2-sum(sum(AtX.*S));
        if cost+regcost<=costold
            stop=1;
            mu_s=mu_s*1.2;
        else
            mu_s=mu_s/2;
        end
    end

    costold=cost;    

    % Update A
    XSt=X*S';   
    gradA=A*SSt-XSt;
    Aold=A;
    stop=0;
    gradA=gradA-repmat(sum(gradA.*A,1),[M,1]).*A;
    while ~stop % Perform linesearch
        A=Aold-mu_a*gradA;
        for k=1:noc
            A(:,k)=A(:,k)/sqrt(sum(A(:,k).^2,1));
        end
        AtA=A'*A;
        cost=SST/2+sum(sum(AtA.*SSt))/2-sum(sum(XSt.*A));
        if cost<=costold
            stop=1;
            mu_a=mu_a*1.2;
        else
            mu_a=mu_a/2;
        end             
    end
    varexp=1-2*cost/SST;
    cost=cost+regcost;    
    dcost=(costold2-cost)/cost;
    disp(sprintf('%12.0f | %12.4f | %2.5e | %2.5e | %12.4f | %2.4e | %2.4e ',iter,varexp,cost,dcost ,toc,mu_a,mu_s));
end

%--------------------------------------------------------------------------
function H=ifft_nyquist(Hf,mod2)
% The inverse fourier transform of a real signal given by frequencies up to
% Nyquist
% Input
%   Hf      noc x F matrix of component frequencies up to the Nyquist
%           criterion
%   mod2    1: even number of samples, 0: odd number of samples
%
% Output
%   H       noc x J matrix in time domain

if ndims(Hf)==2
    if mod2
        Hf=[Hf conj(Hf(:,end-1:-1:2))];
        H=ifft(Hf,[],2,'symmetric');
    else
        Hf=[Hf conj(Hf(:,end:-1:2))];
        H=ifft(Hf,[],2,'symmetric');
    end
elseif ndims(Hf)==3
    if mod2
        Hf=cat(3,Hf,conj(Hf(:,:,end-1:-1:2)));
        H=ifft(Hf,[],3,'symmetric');
    else
        Hf=cat(3,Hf,conj(Hf(:,:,end:-1:2)));
        H=ifft(Hf,[],3,'symmetric');
    end
end


% -------------------------------------------------------------------------
% Parser for optional arguments
function var = mgetopt(opts, varname, default, varargin)
if isfield(opts, varname)
    var = getfield(opts, varname); 
else
    var = default;
end
for narg = 1:2:nargin-4
    cmd = varargin{narg};
    arg = varargin{narg+1};
    switch cmd
        case 'instrset',
            if ~any(strcmp(arg, var))
                fprintf(['Wrong argument %s = ''%s'' - ', ...
                    'Using default : %s = ''%s''\n'], ...
                    varname, var, varname, default);
                var = default;
            end
        otherwise,
            error('Wrong option: %s.', cmd);
    end
end

